# Backend for Neostrada configuration GUI
import sys
import os
import signal
import threading
import subprocess
import tempfile
import re
import unittest
import shutil
import datetime
import time
import traceback
import neogui_lib

logLevel = 10
simulateConnect = False

def englishEnviron():
    env = os.environ
    env["LC_ALL"] = "C"
    return env

def dlog(level, message):
    if level <=logLevel:
        print message

def safeDirectoryRemove(directoryPath):
    print "Removing directory not implemented, path=%s" % directoryPath
    #if directoryPath != "/" and directoryPath != "/tmp" and directoryPath != "/home"
    #   shutil.rmtree()

def runSimpleCommand(command):
    popenObj = subprocess.Popen(
        command,
        bufsize = 1, #0 - unbuffered, 1 - line-buffered, >1 bufsize
        stdout=subprocess.PIPE
        )
    output = popenObj.communicate()
    returncode = popenObj.wait()
    return returncode
def runSimpleCommandWithOutput(command):
    popenObj = subprocess.Popen(
        command,
        bufsize = 1, #0 - unbuffered, 1 - line-buffered, >1 bufsize
        stdout=subprocess.PIPE
        )
    output = popenObj.communicate()
    returncode = popenObj.wait()
    return (returncode, output)

def runSimpleCommandWithTextOutput(command):
    returncode, out = runSimpleCommandWithOutput(command)
    return (returncode, out[0])

def addUserToPapFileContents(username, password, currentContents):
    newContents = []
    usernameCharacters = "[^\s'\"]" #non-whitespace and not quote
    passwordCharacters = "[^\s'\"]" #non-whitespace and not quote
    regex = re.compile(r"^\s*['\"]?(%s+)['\"]?\s+\*\s+['\"]?(%s+)['\"]?\s+\*\s*$" % (usernameCharacters, passwordCharacters) )
    userFound = False
    endOfLine = True
    #filter out line for this user, if it exists
    for line in currentContents:
        if len(line) > 0 and line[-1] == "\n":
            endOfLine = True
        else:
            endOfLine = False
        matchObj = regex.match(line)
        if matchObj == None:
            newContents.append(line)
            continue
        lineUsername = matchObj.group(1)
        linePassword = matchObj.group(2)
        if lineUsername == username:
            userFound = True
            #skip line for this user
        else:
            newContents.append(line)
    if endOfLine == False:
        newContents.append("\n")
    #now add new line for this user
    newContents.append("%s * %s *\n" % (username, password))
    return newContents

#Adds newLine to file contents if it does not contain line matching regexp
#Returns new file contents
def addLineToFileContentsIfNotMatching(regexp, newLine, currentContents):
    newContents = []
    regex = re.compile(regexp)
    lineFound = False
    endOfLine = True
    for line in currentContents:
        dlog(3, "Analyzing line: %s" % line)
        if len(line) > 0 and line[-1] == "\n":
            endOfLine = True
        else:
            endOfLine = False
        matchObj = regex.match(line)
        if matchObj != None:
            dlog(3, "Line %s matches" % line)
            lineFound = True
        else:
            dlog(3, "Line %s does not match" % line)
        newContents.append(line)
    if endOfLine == False:
        newContents.append("\n")
    #now add new line for this user
    if lineFound == False:
        newContents.append(newLine)
    return newContents
    
#Filters out all lines matching given regexp
#Returns new file (contents, replaced), where replaced is True iff something was replaced in file
def filterOutFileContentsMatchingLines(regexp, currentContents):
    newContents = []
    regex = re.compile(regexp)
    lineFound = False
    for line in currentContents:
        dlog(10, "Analyzing line: %s" % line)
        if regex.match(line):
            dlog(10, "Line %s matches, skipping" % line)
            lineFound = True
        else:
            newContents.append(line)
            dlog(10, "Line %s does not match" % line)
    return (newContents, lineFound)
    
def makeFileBackup(filePath, backupsPath = None):
    maxBackups = 200
    if backupsPath == None:
        backupFilePath = filePath
    else:
        backupFilePath = backupsPath + "/" + os.path.basename(filePath)
        
    backupBaseName = backupFilePath + "." + datetime.date.today().isoformat()
        
    for i in range(1, maxBackups):
        backupName = backupBaseName + "." + str(i)
        if os.path.exists(backupName):
            continue
        if not os.path.exists(os.path.dirname(backupName)):
            os.makedirs(os.path.dirname(backupName))
        shutil.copyfile(filePath, backupName)
        return
    raise Exception, "More than %d backups, someone might be trying to do something nasty! Filename=%s" % (maxBackups, filePath)

def writeConfigFile(filePath, fileContents, backupsPath = None):
    """Writes config file, keeps backup of previous file contents.
    """
    if os.path.exists(filePath):
        makeFileBackup(filePath, backupsPath)
    f = open(filePath,"w")
    try:
        f.write(fileContents)
    finally:
        f.close()
        
def readTemplateFile(templateFileName, templateVariables = {}):
    filePath = neogui_lib.thisScriptDirectory() + "/file_templates/" + templateFileName
    f = open(filePath,"r")
    try:
        contents = f.readlines()
    finally:
        f.close()
    #FIXME: replace template variables in lines
    return "".join(contents)
    
def writePapConfigFile(username, password, configFilePath):
    f = open(configFilePath,"r")
    try:
        currentContents = f.readlines()
    finally:
        f.close()
    newContents = addUserToPapFileContents(username, password, currentContents)
    writeConfigFile(configFilePath, "".join(newContents))

class LongActionWrapper:
    def __init__(self, lineOutputFunction, heartbeatFunction, heartbeatIntervalMs = 1000):
        self.lineOutputFunction = lineOutputFunction
        self.heartbeatFunction = heartbeatFunction
        self.heartbeatIntervalMs = heartbeatIntervalMs
    def runCommand(self, command, workingDirectory = None, env = None):
        import subprocess
        popenObj = subprocess.Popen(
            command,
            bufsize = 0, #0 - unbuffered, 1 - line-buffered, >1 bufsize
            stdout=subprocess.PIPE,
            stdin=subprocess.PIPE,
            stderr = subprocess.STDOUT,
            cwd = workingDirectory,
            env = env
            )
        finishedOk = False
        returncode = -999
        try:
            import select
            pollObj = select.poll()
            pollObj.register(popenObj.stdout)
            dots = ""
            heartbeatCounter = 0
            line = ""
            while True:
                dlog(11, "running poll()")
                events = pollObj.poll(self.heartbeatIntervalMs) #milliseconds
                dlog(12, "poll events=%s" % repr(events))
                if len(events) > 0:
                    dlog(11, "read")
                    byte = popenObj.stdout.read(1)
                    dlog(11, "byte=%s" % byte)
                    line += byte
                    dlog(11, "before if")
                    if (len(byte) == 0 and len(line) > 0) or byte == "\n":
                        dlog(11, "ending line")
                        if self.lineOutputFunction(line.decode("utf_8"), popenObj.stdin) == False:
                            dlog(10, "Line output function %s returned false" % repr(self.lineOutputFunction))
                            returncode = -1000
                            return returncode
                        line = ""
                    if len(byte) == 0:
                        dlog(11, "end of file")
                        #eof
                        break
                    dlog(11, "nothing, continue")
                    continue
                else:
                    heartbeatCounter += 1
                    self.heartbeatFunction(heartbeatCounter)
            returncode = popenObj.wait()
            finishedOk = True
        finally:
            if finishedOk == False:
                print "Exception occurred, trying to kill process with TERM, pid=%d" % popenObj.pid
                try:
                    os.kill(popenObj.pid, signal.SIGTERM)
                except OSError:
                    pass
                time.sleep(1.0)
                print "Exception occurred, trying to kill process with KILL, pid=%d" % popenObj.pid
                try:
                    os.kill(popenObj.pid, signal.SIGKILL)
                except OSError:
                    pass
        print "Command %s finished with return code %d" % (repr(command), returncode)
        return returncode

class NeoGuiConfigBackend:
    def __init__(self, frontend):
        self.frontend = frontend
    def startWizard(self):
        #thread1 = threading.Thread(group=None, target=threadfunc, name=None, args=(win), kwargs={})
        thread = threading.Thread(None, self.start, None, ["param1"])
        thread.start()
    def startConnection(self):
        thread = threading.Thread(None, self.connectToNetwork, None, [])
        thread.start()
    def findMountedCds(self, actionWrapper):
        popenObj = subprocess.Popen(
            ["hal-find-by-property", "--key", "volume.fstype", "--string", "iso9660"], #frickin hal-find-by-property works only on strings :/
            bufsize = 1, #0 - unbuffered, 1 - line-buffered, >1 bufsize
            stdout=subprocess.PIPE,
            env = englishEnviron()
            )
        mountPaths = []
        
        output = popenObj.communicate()
        returncode = popenObj.wait()
        dlog(10, "Find CD-rom command finished with return code %d, output:%s\n" % (returncode, output))
        if returncode != 0:
            dlog(2, "hal-find-by-property did not find anything")
            return mountPaths
        
        cdDeviceUdis = output
        for device in cdDeviceUdis:
            if device == None:
                continue
            device = device.rstrip("\n")
            
            popenObj = subprocess.Popen(
                ["hal-get-property", "--udi", device, "--key", "volume.is_mounted" ],
                bufsize = 1, #0 - unbuffered, 1 - line-buffered, >1 bufsize
                stdout=subprocess.PIPE,
                env = englishEnviron()
                )
            if popenObj.communicate()[0].rstrip("\n") == "true":
                mounted = True
            else:
                mounted = False
            
            if mounted == False:
                popenObj = subprocess.Popen(
                    ["hal-get-property", "--udi", device, "--key", "block.device" ],
                    bufsize = 1, #0 - unbuffered, 1 - line-buffered, >1 bufsize
                    stdout=subprocess.PIPE,
                    env = englishEnviron()
                    )
                blockDevice = popenObj.communicate()[0].rstrip("\n")
                if blockDevice != "":
                    self.frontend.performAction("progressInfo", {"text": _("Found unmounted CD, mounting")})
                    self.frontend.performAction("details", {"text": _("Found unmounted CD as device %s, mounting") % blockDevice})
                    returncode = actionWrapper.runCommand(
                        ["mount", blockDevice]
                        )
                    if returncode != 0:
                        self.frontend.performAction("progressInfo", {"text": _("Mounting failed")})
                        self.frontend.performAction("details", {"text": _("Mount device %s failed, error=%d") % (blockDevice, returncode)})
            
            popenObj = subprocess.Popen(
                ["hal-get-property", "--udi", device, "--key", "volume.mount_point" ],
                bufsize = 1, #0 - unbuffered, 1 - line-buffered, >1 bufsize
                stdout=subprocess.PIPE,
                env = englishEnviron()
                )
            output = popenObj.communicate()[0].rstrip("\n")
            returncode = popenObj.wait()
            dlog(10, "Find device command finished with return code %d, output:%s\n" % (returncode, output))
            if returncode != 0:
                dlog(1, "Find device command failed with return code %d, output:%s\n" % (returncode, output))
            else:
                mountPaths.append(output)
        
        return mountPaths
    
    #Tries to locate given file in CD-ROMs and other removable media
    def locateFileOnCdrom(self, fileName, possibleLocations = ""):
        self.frontend.performAction(
            "setNextPage", 
            {
                "nextEnabled": False,
                "pageName": "actionProgress",
                "pageTitle": _("Locating file on CD-ROM"),
                "progressLabel" : _("Please insert CD")
                }
            )
        actionWrapper = LongActionWrapper(
            self.genericLineOutput,
            self.insertCdHeartbeat
            )
        firmwareFilePath = None
        while firmwareFilePath == None:
            mountPaths = self.findMountedCds(actionWrapper)
            for path in mountPaths:
                for possiblePath in possibleLocations:
                    possibleFilePath = path + possiblePath + "/" + fileName
                    if os.path.exists(possibleFilePath):
                        firmwareFilePath = possibleFilePath
                        self.frontend.performAction("progressInfo", {"text": _("Found CD")})
                        self.frontend.performAction("progressInfo", {"details": _("Found CD, path to firmware file: %s") % firmwareFilePath})
                        break
                    else:
                        self.frontend.performAction("progressInfo", {"text": _("CD does not contain firmware, please replace with proper CD")})
                        self.frontend.performAction("progressInfo", {"details": _("Found CD mounted in %s, but it does not contain firmware") % path})
                    self.frontend.raiseIfTerminateRequested()
            time.sleep(0.5)
        return firmwareFilePath
            
    def unpackSpeedtouchFirmwareFromInstallCD(self):
        firmwareFilePath = self.locateFileOnCdrom("neostrada_tp.tgz", ["/Linux/ST330/"])
        if firmwareFilePath == None:
            return False
        self.frontend.performAction(
            "setNextPage", 
            {
                "pageName": "actionProgress",
                "pageTitle": _("Firmware installation"),
                "progressLabel" : _("Installing firmware")
                }
            )
        actionWrapper = LongActionWrapper(
            self.genericLineOutput,
            self.installFirmwareHeartbeat
            )
        self.frontend.performAction("progressInfo", {"text": _("Installing firmware")})
        installCdFirmwareTempDir = tempfile.mkdtemp(dir=self.mainTempDir)
        #uncompress firmware package
        returncode = actionWrapper.runCommand(
            ["tar", "zxf", firmwareFilePath, "-C", installCdFirmwareTempDir]
            )
        if returncode != 0:
            tmpText = _("Uncompressing firmware archive failed")
            self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText})
            return False
        self.frontend.performAction("progressInfo", {"details": _("Firmware package uncompressed")})
        archivePath = installCdFirmwareTempDir + "/neostrada_tp/ST330_firmware_3012.zip"
        if os.path.exists(archivePath) == False:
            tmpText = _("Archive does not contain firmware")
            self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText})
            return False
        self.firmwareFileLocation = archivePath
        return True
    def configureSagemFast800(self):
        if self.prepareDriverCompilation() == False:
            return False
        #unplug modem so that new drivers can be loaded
        #in first run, also previous modem driver eagle_usb can be loaded!
        self.unplugModem("eagle_usb", removeDrivers = True)
        self.unplugModem("ueagle_atm", removeDrivers = True)
        
        if self.installDriver(
                neogui_lib.thisScriptDirectory() + "/tools/ueagle-atm-1.3.tar.gz", 
                neogui_lib.thisScriptDirectory() + "/file_templates/ueagle-compile-initscript",   
                "ueagle-update"
                ) == False:
            return False
        #unpack firmware 
        self.frontend.performAction("progressInfo", {"details": _("Unpacking firmware")})
        returncode = runSimpleCommand(["tar", "zxf", neogui_lib.thisScriptDirectory() + "/tools/ueagle-data-1.1.tar.gz", "-C", "/lib/firmware", "--no-same-owner" ])
        if returncode != 0:
            tmpText = _("Failed to unpack firmware")
            print "Failed to unpack firmware, error=%d" % returncode
            self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
            self.frontend.waitForInput()
            return False
        firmwareDir = "/lib/firmware/ueagle-atm"
        if os.path.exists(firmwareDir):
            self.frontend.performAction("progressInfo", {"details": _("Removing old firmware")})
            returncode = runSimpleCommand(["rm", "-rf", firmwareDir]) 
            print "Removing old firmware, result=%d" % returncode
        #move to proper name
        self.frontend.performAction("progressInfo", {"details": _("Moving firmware to firmware directory")})
        returncode = runSimpleCommand(["mv", "/lib/firmware/ueagle-data-1.1", firmwareDir ]) 
        if returncode != 0:
            tmpText = _("Failed to move firmware")
            print "Failed to move firmware, error=%d" % returncode
            self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
            self.frontend.waitForInput()
            return False
        return True
    
    def configureSagemE4(self):
        if self.prepareDriverCompilation() == False:
            return False
        #unplug modem so that new drivers can be loaded
        #in first run, also previous modem driver eagle_usb can be loaded!
        self.unplugModem("eagle_usb", removeDrivers = True)
        self.unplugModem("ueagle_atm", removeDrivers = True)

	#choose the driver to use
        self.frontend.performAction(
            "setNextPage", 
            {
                "pageName": "simpleChoice",
                "pageTitle" : _("Driver choice"),
                "text": _("<p>Please choose the driver to use. If you do not know which one to choose, leave the default.</p>"),
                "choices": {
                    "old": {"text": _("Standard driver") },
                    "new": {"text": _("New driver"), "checked" : True }
                    }
                }
            )
        output = self.frontend.waitForInput()
        
        if output["choice"] == "old":
                print "Chosen old driver"
                driverPackage = neogui_lib.thisScriptDirectory() + "/tools/ueagle4-atm-old.tgz"
                updateScript = neogui_lib.thisScriptDirectory() + "/file_templates/ueagle4-compile-initscript-old"
        else:
                print "Chosen new driver"
                driverPackage = neogui_lib.thisScriptDirectory() + "/tools/ueagle4-atm.tgz"
                updateScript = neogui_lib.thisScriptDirectory() + "/file_templates/ueagle4-compile-initscript"
        
        if self.installDriver(
                driverPackage, 
                updateScript,   
                "ueagle4-update",
		"ueagle4-atm.tgz"
                ) == False:
            return False
        #unpack firmware 
        self.frontend.performAction("progressInfo", {"details": _("Unpacking firmware")})
        returncode = runSimpleCommand(["tar", "zxf", neogui_lib.thisScriptDirectory() + "/tools/firmware-e4.tgz", "-C", "/lib/firmware", "--no-same-owner" ])
        if returncode != 0:
            tmpText = _("Failed to unpack firmware")
            print "Failed to unpack firmware, error=%d" % returncode
            self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
            self.frontend.waitForInput()
            return False
        return True
        
    def prepareDriverCompilation(self):
        if self.makeSureSoftwareInstalled("build-essential") == False:
            print "Aborting configuration, software not installed"
            return False
        returncode, output = runSimpleCommandWithOutput(["uname", "-r"])
        if returncode != 0:
            print "Running uname -r failed, error=%d" % returncode
        kernelVariant = output[0].rstrip("\n").split("-")[-1]
        print "uname -r=%s kernel variant=%s" % (repr(output), kernelVariant)
        
        linuxHeadersName = "linux-headers-" + kernelVariant
        if self.makeSureSoftwareInstalled(linuxHeadersName) == False:
            print "Aborting configuration, linux headers not installed"
            return False
        return True
        
    def installDriver(self, driverPackageLocation, driverInitScriptLocation, initscriptName, driverPackageNewName = None):
        self.frontend.performAction(
            "setNextPage", 
            {
                "pageName": "actionProgress",
                "pageTitle": _("Drivers installation"),
                "progressLabel" : _("Installing drivers")
                }
            )
        actionWrapper = LongActionWrapper(
            self.genericLineOutput,
            self.installDriversHeartbeat
            )
            
        driversPackageDirectory = "/root/drivers" #Must be on root partition, /root directory should always be there
        self.frontend.performAction("progressInfo", {"details": _("Creating directory for drivers")})
        returncode = runSimpleCommand(["mkdir", "-p", driversPackageDirectory])
        if returncode != 0:
            tmpText = _("Failed to create directory for driver package")
            print "Failed to create package directory, error=%d" % returncode
            self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
            self.frontend.waitForInput()
            return False
        
        self.frontend.performAction("progressInfo", {"details": _("Copying drivers package")})
	if driverPackageNewName != None:
		packageDestination = driversPackageDirectory + "/" + driverPackageNewName
	else:
		packageDestination = driversPackageDirectory + "/"
        returncode = runSimpleCommand(["cp", "-f", driverPackageLocation, packageDestination])
        if returncode != 0:
            tmpText = _("Failed to copy drivers package")
            print "Failed to copy drivers package, error=%d" % returncode
            self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
            self.frontend.waitForInput()
            return False
        #run compilation initscript
        self.frontend.performAction("progressInfo", {"details": _("Installing driver")})
        returncode = actionWrapper.runCommand(
                ["/bin/sh", driverInitScriptLocation, "reinstall"]
                )
        if returncode != 0:
            tmpText = _("Failed to install driver")
            print "Failed to install drivers package, error=%d" % returncode
            self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
            self.frontend.waitForInput()
            return False
        #copy initscript
        self.frontend.performAction("progressInfo", {"details": _("Installing driver update script")})
        returncode = runSimpleCommand(["install", driverInitScriptLocation, "/etc/init.d/" + initscriptName ])
        if returncode != 0:
            tmpText = _("Failed to install driver update script")
            print "Failed to install driver update script, error=%d" % returncode
            self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
            self.frontend.waitForInput()
            return False
        #install initscript boot sequence
        self.frontend.performAction("progressInfo", {"details": _("Updating boot sequence")})
        returncode = runSimpleCommand(["update-rc.d", "-f", initscriptName, "remove"])
        if returncode != 0:
            tmpText = _("Failed to change boot sequence")
            print "Failed to remove previous boot sequence, error=%d" % returncode
            self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
            self.frontend.waitForInput()
            return False
        returncode = runSimpleCommand(["update-rc.d", initscriptName, "defaults", "12" ]) #12 - before connection script starts
        if returncode != 0:
            tmpText = _("Failed to change boot sequence")
            print "Failed to change boot sequence, error=%d" % returncode
            self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
            self.frontend.waitForInput()
            return False
        return True
    def configureZx852v2(self):
        self.makeSureSoftwareInstalled("pppoe_3.5-4ubuntu1_i386.deb", True)
        if self.prepareDriverCompilation() == False:
            return False
            
        #unplug modem so that new drivers can be loaded
        #in first run, also previous modem driver eagle_usb can be loaded!
        driverName = "unicorn_usb_eth"
        self.unplugModem(driverName, removeDrivers = True)
        if self.installDriver(
                neogui_lib.thisScriptDirectory() + "/tools/unicorn-neo.tar.gz",
                neogui_lib.thisScriptDirectory() + "/file_templates/zxdslv2-compile-initscript",   
                "zxdslv2-update"
                ) == False:
            return False
        return True
    def addToDriverBlacklist(self, driverName):
        blacklistFilePath = "/etc/modprobe.d/blacklist"
        f = open(blacklistFilePath,"r")
        try:
            currentContents = f.readlines()
        finally:
            f.close()
        newContents = addLineToFileContentsIfNotMatching(r"^\s*blacklist\s+"+re.escape(driverName)+r"\b", "blacklist " + re.escape(driverName) + "\n", currentContents)
        writeConfigFile(blacklistFilePath, "".join(newContents), "/etc/modprobe.d.backups")
        
    def removeDriverFromBlacklist(self, driverName):
        blacklistFilePath = "/etc/modprobe.d/blacklist"
        f = open(blacklistFilePath,"r")
        try:
            currentContents = f.readlines()
        finally:
            f.close()
        newContents, replaced = filterOutFileContentsMatchingLines(r"^\s*blacklist\s+"+re.escape(driverName)+r"\b", currentContents)
        if replaced == True:
            writeConfigFile(blacklistFilePath, "".join(newContents), "/etc/modprobe.d.backups")
        return replaced
        
    def configureZx852(self):
        driverName = "cxacru"
        #workaround for cxacru bug preventing unload of loaded module
        blacklistDriver = False
        if self.checkModuleLoaded(driverName) == True:
            dlog(10, "Driver %s loaded" % driverName)
            returncode, output = runSimpleCommandWithOutput(["modinfo", "-F", "filename", driverName])
            if returncode != 0:
                print "Running modinfo failed, error=%d" % returncode
                return False
            out = output[0]
            dlog(10, "modinfo output: %s" % out)
            if re.match(r"/lib/modules/\S+/kernel/drivers/usb/atm/", out):
                blacklistDriver = True

        if blacklistDriver == True:
            dlog(1, "Adding cxacru to blacklist")
            self.frontend.performAction(
                "setNextPage", 
                {
                    "pageName": "actionProgress",
                    "pageTitle": _("Blacklisting broken driver"),
                    "progressLabel" : _("Blacklisting broken driver")
                    }
                )
            actionWrapper = LongActionWrapper(
                self.genericLineOutput,
                self.installFirmwareHeartbeat
                )
            self.frontend.performAction("progressInfo", {"text": _("Adding driver to blacklist")})
            self.addToDriverBlacklist(driverName)
            self.frontend.performAction("progressInfo", {"text": _("Recreating driver database")})
            returncode = actionWrapper.runCommand(
                ["/sbin/depmod", "-ae"],
                )
            self.frontend.performAction(
                "setNextPage", 
                {
                    "pageName": "simpleMessage",
                    "pageTitle" : _("Modem warning"),
                    "nextEnabled" : False,
                    "finishEnabled" : True,
                    "text": _("<p>Due to bug in driver distributed with Ubuntu, modem driver cannot be removed once it was loaded. Wizard prevented driver from being loaded during next boot. Please reboot and run the wizard again.</p>"),
                    }
                )
            self.frontend.waitForInput()
            return False
            
        if self.prepareDriverCompilation() == False:
            return False
            
        #unplug modem so that new drivers can be loaded
        #in first run, also previous modem driver eagle_usb can be loaded!
        self.unplugModem(driverName, removeDrivers = True)
        if self.installDriver(
                neogui_lib.thisScriptDirectory() + "/tools/usbatm-zxdsl852.tar.gz", 
                neogui_lib.thisScriptDirectory() + "/file_templates/zxdsl-compile-initscript",   
                "zxdsl-update"
                ) == False:
            return False
        dlog(1, "Removing driver from blacklist")
        if self.removeDriverFromBlacklist(driverName):
            dlog(1, "Running depmod")
            returncode = runSimpleCommand(
                ["/sbin/depmod", "-ae"],
                )
            if returncode != 0:
                dlog(1, "Running depmod failed")
                return False
        #now firmware
        self.frontend.performAction("setNextPage", {"pageName": "firmwareInstallationSource"})
        firmwareInstallationSource = self.frontend.waitForInput()
        source = firmwareInstallationSource["firmwareSource"]
        print "Firmware source: %s" % source
        if source == "location":
            self.frontend.performAction("setNextPage", {"pageName": "firmwareInstallationFromLocation", "firmwareFileWildcard" : "cxacru-fw.bin"})
            firmwareFileLocationOutput = self.frontend.waitForInput()
            firmwareFileLocation = firmwareFileLocationOutput["firmwareFileLocation"]
            
            self.frontend.performAction(
                "setNextPage", 
                {
                    "pageName": "actionProgress",
                    "pageTitle": _("Firmware installation"),
                    "progressLabel" : _("Installing firmware")
                    }
                )
            actionWrapper = LongActionWrapper(
                self.genericLineOutput,
                self.installFirmwareHeartbeat
                )
            self.frontend.performAction("progressInfo", {"text": _("Installing firmware")})
            returncode = actionWrapper.runCommand(
                ["cp", firmwareFileLocation, "/lib/firmware"],
                )
            if returncode != 0:
                tmpText = _("Installing firmware failed")
                self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
                self.frontend.waitForInput()
                return False
            self.frontend.performAction("progressInfo", {"details": _("Firmware installed")})
        elif source == "InstallCD":
            if self.unpackZxdsl852FirmwareFromInstallCD() == False:
                print "Unpacking firmware from install CD failed"
                return False
        else:
            raise Exception, "Unknown firmware source: %s " % source
        
        return True
        
    def unpackZxdsl852FirmwareFromInstallCD(self):
        self.firmwareFileLocation = self.locateFileOnCdrom(
            "CnxEtU.sys", 
            ["/NeostradaTP/Drivers/ZTEZXDSL852"]
            )
        if self.firmwareFileLocation == None:
            print "Cannot locate CD with firmware"
            return False

        print "Firmware file location=%s" % self.firmwareFileLocation
        if self.makeSureSoftwareInstalled("build-essential") == False:
            print "Aborting configuration, software not installed"
            return False
        self.frontend.performAction(
            "setNextPage", 
            {
                "pageName": "actionProgress",
                "pageTitle": _("Firmware installation"),
                "progressLabel" : _("Installing firmware")
                }
            )
        actionWrapper = LongActionWrapper(
            self.genericLineOutput,
            self.installFirmwareHeartbeat
            )
        self.frontend.performAction("progressInfo", {"text": _("Installing firmware")})
        self.frontend.performAction("progressInfo", {"details": _("Uncompressing firmware extractor utility")})
        firmwareUnpackTempDir = tempfile.mkdtemp(dir=self.mainTempDir)
        try:
            firmwareExtractorPackagePath = neogui_lib.thisScriptDirectory() + "/tools/cxacru-fw-ng.tar.gz"
            #uncompress firmware extractor utility
            returncode = actionWrapper.runCommand(
                ["tar", "zxf", firmwareExtractorPackagePath, "-C", firmwareUnpackTempDir]
                )
            if returncode != 0:
                tmpText = _("Uncompressing firmware extractor utility failed")
                self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
                self.frontend.waitForInput()
                return False
            self.frontend.performAction("progressInfo", {"details": _("Firmware extractor utility uncompressed")})
            firmwareExtractorDirectory = firmwareUnpackTempDir
            print "firmwareExtractorDirectory=%s" % firmwareExtractorDirectory
            #compile firmware extractor
            returncode = actionWrapper.runCommand(
                ["make" ],
                workingDirectory = firmwareExtractorDirectory
                )
            if returncode != 0:
                tmpText = _("Compiling firmware extractor failed")
                self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
                self.frontend.waitForInput()
                return False
            #install firmware
            returncode = actionWrapper.runCommand(
                ["./cxacru-fw", self.firmwareFileLocation,  "cxacru-fw.bin"],
                workingDirectory = firmwareExtractorDirectory
                )
            if returncode != 0:
                tmpText = _("Extracting firmware failed")
                self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
                self.frontend.waitForInput()
                return False
            firmwareFileLocation = firmwareExtractorDirectory + "/cxacru-fw.bin"
            self.frontend.performAction("progressInfo", {"details": _("Firmware extracted")})
            returncode = actionWrapper.runCommand(
                ["cp", firmwareFileLocation, "/lib/firmware/"],
                )
            if returncode != 0:
                tmpText = _("Copying firmware file failed")
                self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
                self.frontend.waitForInput()
                return False
            self.frontend.performAction("progressInfo", {"details": _("Firmware file copied")})
        finally:
            safeDirectoryRemove(firmwareUnpackTempDir)
        
    def configureSpeedtouch(self):
        self.frontend.performAction("setNextPage", {"pageName": "firmwareInstallationSource"})
        firmwareInstallationSource = self.frontend.waitForInput()
        source = firmwareInstallationSource["firmwareSource"]
        print "Firmware source: %s" % source
        if source == "location":
            self.frontend.performAction("setNextPage", {"pageName": "firmwareInstallationFromLocation", "firmwareFileWildcard" : "SpeedTouch330_firmware_*.zip"})
            firmwareFileLocationOutput = self.frontend.waitForInput()
            self.firmwareFileLocation = firmwareFileLocationOutput["firmwareFileLocation"]
        elif source == "InstallCD":
            if self.unpackSpeedtouchFirmwareFromInstallCD() == False:
                print "Unpacking firmware from install CD failed"
                return False
        else:
            raise Exception, "Unknown firmware source: %s " % source
        print "Firmware file location=%s" % self.firmwareFileLocation
        if self.makeSureSoftwareInstalled("build-essential") == False:
            print "Aborting configuration, software not installed"
            return False
        if self.makeSureSoftwareInstalled("unzip") == False:
            print "Aborting configuration, software not installed"
            return False
        self.replugModem("speedtch")
        self.frontend.performAction(
            "setNextPage", 
            {
                "pageName": "actionProgress",
                "pageTitle": _("Firmware installation"),
                "progressLabel" : _("Installing firmware")
                }
            )
        actionWrapper = LongActionWrapper(
            self.genericLineOutput,
            self.installFirmwareHeartbeat
            )
        self.frontend.performAction("progressInfo", {"text": _("Installing firmware")})
        self.frontend.performAction("progressInfo", {"details": _("Checking firmware version")})
        output = subprocess.Popen(
            ["grep", "\<ProdID=4061\>", "/proc/bus/usb/devices"],
            stdout=subprocess.PIPE, 
            env = englishEnviron()
            ).communicate()[0]
        words = output.split()
        if len(words) == 0: #or (words[-1] != "4.00" and words[-1] != "2.00")
            tmpText = _("Cannot check firmware version, is modem connected?")
            self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
            self.frontend.waitForInput()
            return False
        
        firmwareVersion = words[-1]
        self.frontend.performAction("progressInfo", {"details": _("Detected firmware version %s") % firmwareVersion})
        
        self.frontend.performAction("progressInfo", {"details": _("Uncompressing firmware extractor utility")})
        firmwareUnpackTempDir = tempfile.mkdtemp(dir=self.mainTempDir)
        try:
            firmwareExtractorPackagePath = neogui_lib.thisScriptDirectory() + "/tools/firmware-extractor.tar.gz"
            #uncompress firmware extractor utility
            returncode = actionWrapper.runCommand(
                ["tar", "zxf", firmwareExtractorPackagePath, "-C", firmwareUnpackTempDir]
                )
            if returncode != 0:
                tmpText = _("Uncompressing firmware extractor utility failed")
                self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
                self.frontend.waitForInput()
                return False
            self.frontend.performAction("progressInfo", {"details": _("Firmware extractor utility uncompressed")})
            firmwareExtractorDirectory = firmwareUnpackTempDir + "/firmware-extractor"
            print "firmwareExtractorDirectory=%s" % firmwareExtractorDirectory
            #uncompress firmware package
            returncode = actionWrapper.runCommand(
                ["unzip", self.firmwareFileLocation, "-d", firmwareUnpackTempDir]
                )
            if returncode != 0:
                tmpText = _("Uncompressing firmware package failed")
                self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
                self.frontend.waitForInput()
                return False
            self.frontend.performAction("progressInfo", {"details": _("Firmware package uncompressed")})
            #copy proper firmware as mgmt.o
            if firmwareVersion == "4.00":
                firmwareSourceFileName = "ZZZL_3.012"
            else:
                firmwareSourceFileName = "KQD6_3.012"
            returncode = actionWrapper.runCommand(
                ["cp", firmwareUnpackTempDir + "/" + firmwareSourceFileName, firmwareExtractorDirectory +"/mgmt.o"],
                )
            if returncode != 0:
                tmpText = _("Copying firmware file failed")
                self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
                self.frontend.waitForInput()
                return False
            self.frontend.performAction("progressInfo", {"details": _("Firmware file copied")})
            #configure firmware extractor
            returncode = actionWrapper.runCommand(
                [ firmwareExtractorDirectory + "/configure" ],
                workingDirectory = firmwareExtractorDirectory
                )
            if returncode != 0:
                tmpText = _("Configuring firmware extractor failed")
                self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
                self.frontend.waitForInput()
                return False
            self.frontend.performAction("progressInfo", {"details": _("Firmware extractor configured")})
            #extract firmware
            returncode = actionWrapper.runCommand(
                ["make" ],
                workingDirectory = firmwareExtractorDirectory
                )
            if returncode != 0:
                tmpText = _("Running firmware extractor failed")
                self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
                self.frontend.waitForInput()
                return False
            #install firmware
            returncode = actionWrapper.runCommand(
                ["make", "install"],
                workingDirectory = firmwareExtractorDirectory
                )
            if returncode != 0:
                tmpText = _("Installing firmware failed")
                self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
                self.frontend.waitForInput()
                return False
        finally:
            safeDirectoryRemove(firmwareUnpackTempDir)
        self.frontend.performAction("progressInfo", {"details": _("Firmware extracted")})
    def packageInstalled(self, packageName):
        #try:
        output = subprocess.Popen(
            ["dpkg-query", "-W", "-f${Status}\n", packageName], 
            stdout=subprocess.PIPE, 
            env = englishEnviron()
            ).communicate()[0]
        #except OSError, e:
            #print >>sys.stderr, "Execution failed:", e
        print "output = %s" % repr(output)
        print "output.split() = %s" % repr(output.split())
        installed = "installed" in output.split()
        return installed
    def packageAvailableInRepositories(self, packageName):
        output = subprocess.Popen(
            ["apt-cache", "madison", packageName], 
            stdout=subprocess.PIPE, 
            env = englishEnviron()
            ).communicate()[0]
        print "madison output = %s" % repr(output)
        print "madison output.split() = %s" % repr(output.split("\n"))
        lines = output.split("\n")
        for line in lines:
            words = line.split()
            if len(words) == 0:
                continue
            if words[-1] == "Packages":
                #package is in one of the repositories
                return True
        return False
    def addMainRepository(self):
        while True:
            self.frontend.performAction(
                "setNextPage", 
                {
                    "pageName": "simpleMessage",
                    "pageTitle" : _("Adding CD-ROM with software"),
                    "nextEnabled" : True,
                    "text": _("<p>Please prepare CD-ROM which you used to install your system and click \"Next\".</p>"),
                    }
                )
            self.frontend.waitForInput()
            self.frontend.performAction(
                "setNextPage", 
                {
                    "pageName": "actionProgress",
                    "pageTitle": _("Adding CD with software"),
                    "progressLabel" : _("Adding CD")
                    }
                )
            actionWrapper = LongActionWrapper(
                self.installPackageLineOutput,
                self.addCdHeartbeat
                )
            dlog(10, "Running apt-cdrom add")
            returncode = actionWrapper.runCommand(
                ["apt-cdrom", "add"],
                env = englishEnviron()
                )
            if returncode != 0:
                tmpText = _("Adding CD failed")
                self.frontend.performAction("progressInfo", {"status": False, "nextEnabled" : True, "text": tmpText, "details": tmpText})
                self.frontend.waitForInput()
                return False
            return True
    def makeSureSoftwareInstalled(self, packageName, fromBundledDebs = False):
        self.frontend.performAction(
            "setNextPage", 
            {
                "pageName": "actionProgress",
                "pageTitle": _("Installing software"),
                "progressLabel" : _("Installing software")
                }
            )
        print "Checking availability of package %s" % packageName
        self.frontend.performAction("progressInfo", {"text": _("Checking for availability of necessary software")})
        realPackageName = packageName
        if fromBundledDebs == True:
            realPackageName = self.getDebPackageName(packageName)
            if realPackageName == None:
                print "Cannot find the name of deb package %s failed" % packageName
                self.frontend.performAction("progressInfo", {"status": False, "text": _("Failed to install software")})
                return False
        
        installed = self.packageInstalled(realPackageName)
        if installed == True:
            print "Package %s is installed" % packageName
            return True
        
        if fromBundledDebs == True:
            if self.installBundledDebPackage(packageName) == True:
                print "Bundled package %s has been installed" % packageName
                self.frontend.performAction("progressInfo", {"status": True, "text": _("Software installed successfully")})
                return True
            else:
                print "Installation of bundled package %s failed, trying to install from repository" % packageName
        
        if self.packageAvailableInRepositories(packageName) == False:
            print "Package %s is not available in repositories, proceeding to adding repository" % packageName
            if self.addMainRepository() == False:
                print "Failed to add main repository"
                return False
            print "Main repository added successfully"
        else:
            print "Package %s is available in repositories, proceeding to installation" % packageName
        if self.installPackage(packageName) == True:
            print "Package %s has been installed" % packageName
            self.frontend.performAction("progressInfo", {"status": True, "text": _("Software installed successfully")})
        else:
            print "Installation of package %s failed" % packageName
            self.frontend.performAction("progressInfo", {"status": False, "text": _("Failed to install software")})
            return False
        return True
    def installPackageLineOutput(self, line, stdin):
        dlog(10, "Entered installPackageLineOutput, line=%s" % line)
#        if line.find("Please insert a Disc in the drive and press enter") >= 0:
        if line.find("Waiting for disc") >= 0 or line.find("Please insert the disc labeled") >= 0:
            self.frontend.performAction(
                "showPopupDialog", 
                {
                    "message": _("Please insert installation CD"),
                    "caption": _("Insert CD"),
                    "button0Text" : _("OK, CD is in the drive"),
                    "button1Text" : _("Cancel inserting CD")
                    }
                )
            output = self.frontend.waitForInput()
            if output["result"] == 0:
                #OK
                dlog(3, "OK pressed, sending Enter to process")
                stdin.write("\n")
            else:
                tmpText = _("Cancelled CD insertion upon user request")
                self.frontend.performAction("progressInfo", {"status": False, "text" : tmpText, "details": tmpText})
                dlog(1, "User cancelled CD insertion")
                self.frontend.waitForInput()
                return False
        self.frontend.performAction("progressInfo", {"details": line})
        return True
    def installPackageHeartbeat(self, counter):
        self.frontend.raiseIfForceTerminateRequested()
        dots = ""
        for i in range(0, counter % 4):
            dots += "."
        self.frontend.performAction("progressInfo", {"text": _("Installing") + dots })
    def genericLineOutput(self, line, stdin):
        self.frontend.performAction("progressInfo", {"details": line})
        return True
    def installFirmwareHeartbeat(self, counter):
        self.frontend.raiseIfForceTerminateRequested()
        dots = ""
        for i in range(0, counter % 4):
            dots += "."
        self.frontend.performAction("progressInfo", {"text": _("Please wait") + dots })
    def insertCdHeartbeat(self, counter):
        self.frontend.raiseIfForceTerminateRequested()
        dots = ""
        for i in range(0, counter % 4):
            dots += "."
        self.frontend.performAction("progressInfo", {"text": _("Waiting") + dots })
    def installDriversHeartbeat(self, counter):
        self.frontend.raiseIfForceTerminateRequested()
        dots = "." * (counter % 4)
        self.frontend.performAction("progressInfo", {"text": _("Please wait") + dots })
    def addCdHeartbeat(self, counter):
        self.frontend.raiseIfTerminateRequested()
        dots = "." * (counter % 4)
        self.frontend.performAction("progressInfo", {"text": _("Please wait") + dots })
    def connectToNetworkLineOutput(self, line, stdin):
        self.frontend.performAction("progressInfo", {"details": line})
        return True
    def connectToNetworkHeartbeat(self, counter):
        self.frontend.raiseIfTerminateRequested()
        dots = ""
        for i in range(0, counter % 4):
            dots += "."
        self.frontend.performAction("progressInfo", {"text": _("Please wait") + dots })
    def installPackage(self, packageName):
        self.frontend.performAction("progressInfo", {"text": _("Installing necessary software...")})
        
        actionWrapper = LongActionWrapper(
            self.installPackageLineOutput,
            self.installPackageHeartbeat
            )
        
        returncode = actionWrapper.runCommand(
            ["apt-get", "install", "--assume-yes", packageName, "-o", "APT::Status-Fd=1"],
            env = englishEnviron()
            )
        
        if returncode == 0:
            print "Package %s has been installed successfully" % packageName
        else:
            print "Installation of package %s failed with error %d" % (packageName, returncode)
            return False
        return True
        
    def getBundledDebPackagePath(self, packageName):
        bundledPackagesPath = neogui_lib.thisScriptDirectory() + "/debs"
        return bundledPackagesPath + "/" + packageName
    
    def getDebPackageName(self, packageName):
        returncode, output = runSimpleCommandWithOutput(["dpkg", "-f", self.getBundledDebPackagePath(packageName), "Package"])
        print "dpkg -f output: %s" % repr(output)
        if returncode != 0:
            raise Exception, "Running dpkg -f %s, error=%d output=%s" % (packageName,returncode, repr(output))
        
        return output[0].split("\n")[0]
        
    def installBundledDebPackage(self, packageName):
        self.frontend.performAction("progressInfo", {"text": _("Installing necessary software...")})
        
        actionWrapper = LongActionWrapper(
            self.installPackageLineOutput,
            self.installPackageHeartbeat
            )
        
        returncode = actionWrapper.runCommand(
            ["dpkg", "--status-fd", "1", "-i", self.getBundledDebPackagePath(packageName)],
            env = englishEnviron()
            )
        
        if returncode == 0:
            print "Package %s has been installed successfully" % packageName
        else:
            print "Installation of package %s failed with error %d" % (packageName, returncode)
            return False
        return True
    def waitForSagemModemSync(self, atmProcPath):
        self.frontend.performAction(
            "setNextPage", 
            {
                "pageName": "actionProgress",
                "pageTitle": _("Waiting for modem to synchronize"),
                "progressLabel" : _("Waiting for modem")
                }
            )
        self.frontend.performAction("progressInfo", {"text": _("Waiting for modem")})
        
        waitTime = 60.0
        waitInterval = 0.2
        modemSynchronized = False
        
        for i in range(1, int(waitTime / waitInterval)):
            #find procEntry
            if os.path.exists(atmProcPath):
                return self.waitForAtmModemSync(atmProcPath)
            
            if not os.path.exists("/proc/driver/ueagle-atm"):
                self.frontend.raiseIfTerminateRequested()
                time.sleep(waitInterval)
                continue
            files = os.listdir("/proc/driver/ueagle-atm")
            dlog(10, "files in /proc/driver/ueagle-atm: %s" % repr(files))
            if len(files) > 1:
                dlog(1, "files in /proc/driver/ueagle-atm: %s" % repr(files))
                tmpText = _("More than one modem detected, please disconnect one")
                self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText})
            elif len(files) == 0:
                tmpText = _("No modem detected, waiting")
                self.frontend.performAction("progressInfo", {"details": tmpText})
            else:
                procEntry = "/proc/driver/ueagle-atm/" + files[0]
                dlog(10, "Using proc entry: %s" % procEntry)
                output = subprocess.Popen(
                    ["grep", "^Modem is ", procEntry],
                    stdout=subprocess.PIPE, 
                    env = englishEnviron()
                    ).communicate()[0]
                if output == "Modem is operational\n":
                    self.frontend.performAction("progressInfo", {"text": _("Modem ready")})
                    modemSynchronized = True
                    break
                else:
                    self.frontend.performAction("progressInfo", {"text": _("Modem not ready")})
                    modemSynchronized = False
                    if simulateConnect == True:
                        self.frontend.performAction("progressInfo", {"text": "Simulating fake connect"})
                        modemSynchronized = True
                        break
            self.frontend.raiseIfTerminateRequested()
            time.sleep(waitInterval)
        
        if modemSynchronized == False:
            tmpText = _("Modem failed to synchronize or line is disconnected")
            self.frontend.performAction("progressInfo", {"status": False, "nextEnabled" : True, "text": tmpText, "details": tmpText})
            self.frontend.waitForInput()
            return False
        self.frontend.performAction("progressInfo", {"status": True, "details": _("Modem running")})
        return True
                
    def waitForZx852v2ModemSync(self):
        procFileName = "/proc/net/atm/UNICORN:0"
        self.frontend.performAction(
            "setNextPage", 
            {
                "pageName": "actionProgress",
                "pageTitle": _("Waiting for modem to synchronize"),
                "progressLabel" : _("Waiting for modem")
                }
            )
        self.frontend.performAction("progressInfo", {"text": _("Waiting for modem")})
        if simulateConnect == True:
            self.frontend.performAction("progressInfo", {"text": "Simulating fake connect"})
            modemSynchronized = True
            return True
        
        waitTime = 60.0
        waitInterval = 0.2
        modemSynchronized = False
        
        for i in range(1, int(waitTime / waitInterval)):
            output = subprocess.Popen(
                ["grep", "^ADSL: status", procFileName],
                stdout=subprocess.PIPE, 
                env = englishEnviron()
                ).communicate()[0]
            if output.find("ATM ready") >=0:
                modemSynchronized = True
                break
            elif output.find("unknown") >=0:
                self.frontend.performAction("progressInfo", {"text": _("Line state unknown")})
            elif output == "no link":
                self.frontend.performAction("progressInfo", {"text": _("Line down")})
            elif output.find("no hardware") >=0:
                self.frontend.performAction("progressInfo", {"text": _("No hardware")})
                modemSynchronized = False
                if simulateConnect == True:
                    self.frontend.performAction("progressInfo", {"text": "Simulating fake connect"})
                    modemSynchronized = True
                    break
            self.frontend.raiseIfTerminateRequested()
            time.sleep(waitInterval)
        
        if modemSynchronized == False:
            tmpText = _("Modem failed to synchronize or line is disconnected")
            self.frontend.performAction("progressInfo", {"status": False, "nextEnabled" : True, "text": tmpText, "details": tmpText})
            self.frontend.waitForInput()
            return False
        self.frontend.performAction("progressInfo", {"status": True, "details": _("Modem running")})
        return True
        
    def waitForAtmModemSync(self, procFileName):
        self.frontend.performAction(
            "setNextPage", 
            {
                "pageName": "actionProgress",
                "pageTitle": _("Waiting for modem to synchronize"),
                "progressLabel" : _("Waiting for modem")
                }
            )
        self.frontend.performAction("progressInfo", {"text": _("Waiting for modem")})
        
        waitTime = 60.0
        waitInterval = 0.2
        modemSynchronized = False
        
        for i in range(1, int(waitTime / waitInterval)):
            output = subprocess.Popen(
                ["grep", "^Line ", procFileName],
                stdout=subprocess.PIPE, 
                env = englishEnviron()
                ).communicate()[0]
            if output == "Line up\n":
                modemSynchronized = True
                break
            elif output == "Line state unknown\n":
                self.frontend.performAction("progressInfo", {"text": _("Line state unknown")})
            if output == "Line down\n":
                self.frontend.performAction("progressInfo", {"text": _("Line down")})
                modemSynchronized = False
                if simulateConnect == True:
                    self.frontend.performAction("progressInfo", {"text": "Simulating fake connect"})
                    modemSynchronized = True
                    break
            self.frontend.raiseIfTerminateRequested()
            time.sleep(waitInterval)
        
        if modemSynchronized == False:
            tmpText = _("Modem failed to synchronize or line is disconnected")
            self.frontend.performAction("progressInfo", {"status": False, "nextEnabled" : True, "text": tmpText, "details": tmpText})
            self.frontend.waitForInput()
            return False
        self.frontend.performAction("progressInfo", {"status": True, "details": _("Modem running")})
        return True
    def checkModuleLoaded(self, driverName):
        returncode, output = runSimpleCommandWithOutput(["lsmod"])
        if returncode != 0:
            raise Exception, "Running lsmod failed, error=%d" % returncode
        dlog(10, "lsmod output=%s" % repr(output))
        for line in output[0].split("\n"):
            name = line.split(" ")[0]
            if name == driverName:
                return True
        return False
    def checkDeviceLoaded(self, driverName):
        return self.checkModuleLoaded(driverName)
        
    def unplugModem(self, modemDriverName, removeDrivers = False):
        loopCount = 0
        while True:
            loopCount += 1
            if removeDrivers == True:
                #unload modules so that we have clean state
                returncode = runSimpleCommand(["modprobe", "-r", modemDriverName])
                if returncode == 0:
                    dlog(3, "rmmod %s suceeded" % modemDriverName)
                    returncode = runSimpleCommand(["modprobe", "-r", "usbatm"])
                    if returncode != 0:
                        dlog(3, "rmmod usbatm failed with code %d" % returncode)
                    else:
                        dlog(3, "rmmod usbatm succeeded")
                else:
                    dlog(3, "rmmod %s failed with code %d" % (modemDriverName, returncode))
                if self.checkModuleLoaded(modemDriverName) == False:
                    break
            else:
                if self.checkDeviceLoaded(modemDriverName) == False:
                    break
            if loopCount == 1:
                self.frontend.performAction(
                    "setNextPage", 
                    {
                        "pageName": "unplugModem",
                        "nextEnabled": False
                        }
                    )
            self.frontend.raiseIfTerminateRequested()
            time.sleep(0.2)
    def detectModemDevice(self):
        returncode, output = runSimpleCommandWithTextOutput(["lsusb"])
        if returncode != 0:
            dlog(1, "lsusb failed code=%d" %  returncode)
            return None
        
        #FIXME: use USB device numbers
        if re.compile(re.escape('Analog Devices Canada, Ltd (Allied Telesyn) ')).search(output):
            if re.compile(re.escape('1110:9042 Analog Devices Canada, Ltd (Allied Telesyn) ')).search(output) or re.compile(re.escape('1110:9041 Analog Devices Canada, Ltd (Allied Telesyn) ')).search(output):
                return "sagemE4"
            else:
                return "sagemFast800"
        if re.compile(re.escape('Alcatel Telecom SpeedTouch ISDN or ADSL Modem')).search(output):
            return "speedTouch"
        if re.compile(re.escape('Conexant Systems (Rockwell), Inc.')).search(output):
            return "Zx"
        if re.compile(re.escape('SGS Thomson Microelectronics')).search(output):
            return "Zx2"
        return None
        
    def detectModem(self):
        detectedModem = None
        self.frontend.performAction(
            "setNextPage", 
            {
                "pageName": "plugModem",
                "nextEnabled": True,
                "message": _("If the modem is not detected, please click \"Next\" to choose modem manually.")
                }
            )
        while True:
            detectedModem = self.detectModemDevice()
            if detectedModem != None:
                break
            output = self.frontend.waitForInput(nonBlocking = True)
            if output != None:
                print "User requested manual modem selection"
                break
            self.frontend.raiseIfTerminateRequested()
            time.sleep(0.2)
        return detectedModem
    def plugModem(self, modemDriverName):
        if self.checkDeviceLoaded(modemDriverName) == True:
            return
        self.frontend.performAction(
            "setNextPage", 
            {
                "pageName": "plugModem",
                "nextEnabled": False
                }
            )
        while True:
            if self.checkDeviceLoaded(modemDriverName) == True:
                break
            self.frontend.raiseIfTerminateRequested()
            time.sleep(0.2)
    def replugModem(self, modemDriverName, removeDrivers = True):
        self.unplugModem(modemDriverName, removeDrivers = removeDrivers)
        self.plugModem(modemDriverName)
        
    def setupPppConnection(self, username, password, pppConnectionContents):
        resolvConfContents = """
nameserver 194.204.152.34 
nameserver 217.98.63.164
"""
        writeConfigFile("/etc/ppp/peers/neostrada", pppConnectionContents % username)
        writeConfigFile("/etc/resolv.conf", resolvConfContents)
        writePapConfigFile(username, password, "/etc/ppp/pap-secrets")
        writePapConfigFile(username, password, "/etc/ppp/chap-secrets")
    def setupConnectionStartupAtBoot(self):
        self.frontend.performAction(
            "setNextPage", 
            {
                "pageName": "actionProgress",
                "pageTitle": _("Setting up connection at boot"),
                "progressLabel" : _("Setting up connection at boot")
                }
            )
        self.frontend.performAction("progressInfo", {"text": _("Copying configuration files")})
        self.frontend.performAction("progressInfo", {"details": _("Installing startup script")})
        initScript = readTemplateFile("speedtouch-initscript")
        try:
            writeConfigFile("/etc/init.d/neostrada", initScript)
        except IOError:
            tmpText = _("Failed to install boot startup script")
            self.frontend.performAction("progressInfo", {"status": False, "nextEnabled" : True, "text": tmpText, "details": tmpText})
            self.frontend.waitForInput()
            return False
        
        self.frontend.performAction("progressInfo", {"details": _("Changing permissions of startup script")})
        returncode = runSimpleCommand(
            ["chmod", "+x", "/etc/init.d/neostrada"]
            )
        if returncode != 0:
            tmpText = _("Failed to change permissions of init script")
            self.frontend.performAction("progressInfo", {"status": False, "nextEnabled" : True, "text": tmpText, "details": tmpText})
            self.frontend.waitForInput()
            return False
        self.frontend.performAction("progressInfo", {"details": _("Updating boot sequence")})
        returncode = runSimpleCommand(["update-rc.d", "-f", "neostrada", "remove"])
        if returncode != 0:
            tmpText = _("Failed to change boot sequence")
            print "Failed to remove previous boot sequence, error=%d" % returncode
            self.frontend.performAction("progressInfo", {"text": tmpText, "details": tmpText, "status": False})
            self.frontend.waitForInput()
            return False
        returncode = runSimpleCommand(
            ["update-rc.d", "neostrada", "defaults"]
            )
        if returncode != 0:
            tmpText = _("Failed to change boot sequence")
            self.frontend.performAction("progressInfo", {"status": False, "nextEnabled" : True, "text": tmpText, "details": tmpText})
            self.frontend.waitForInput()
            return False
        self.frontend.performAction("progressInfo", {"status": True, "details": _("Boot startup setup complete")})
        return True
        
    def askForConnectionStartupAtBoot(self):
        self.frontend.performAction(
            "setNextPage", 
            {
                "pageName": "simpleChoice",
                "pageTitle" : _("Persistent connection"),
                "text": _("<p>Do you want to start Neostrada at boot?</p><p>If you choose not to start at boot, you can connect manually using \"Connect\" option in NeoGui</p>"),
                "choices": {
                    "start": {"text": _("Start at boot"), "checked" : True },
                    "dontstart": {"text": _("Do not start at boot") }
                    }
                }
            )
        output = self.frontend.waitForInput()
        
        if output["choice"] == "start":
            print "Chosen to start at boot"
            return self.setupConnectionStartupAtBoot()
        else:
            print "Chosen not to start at boot"
            return True
    def connectToNetwork(self):
        self.frontend.performAction(
            "setNextPage", 
            {
                "pageName": "actionProgress",
                "pageTitle": _("Connecting to network"),
                "progressLabel" : _("Connecting")
                }
            )
        actionWrapper = LongActionWrapper(
            self.connectToNetworkLineOutput,
            self.connectToNetworkHeartbeat
            )
        self.frontend.performAction("progressInfo", {"text": _("Connecting")})
        
        returncode = actionWrapper.runCommand(
            ["pppd", "call", "neostrada"]
            )
        if returncode != 0:
            if simulateConnect == True:
                self.frontend.performAction("progressInfo", {"text": "Simulating fake connect"})
            else:
                tmpText = _("Modem failed to connect")
                self.frontend.performAction("progressInfo", {"status": False, "nextEnabled" : True, "text": tmpText, "details": tmpText})
                return False
        self.frontend.performAction("progressInfo", {"status": True, "details": _("Modem connected")})
        return True
    def connectForRegistration(self, pppConnectionContents):
        self.setupPppConnection("rejestracja@neostrada.pl", "rejestracja", pppConnectionContents)
        return self.connectToNetwork()
    def start(self, params):
        self.mainTempDir = tempfile.mkdtemp()
        try:
            try:
                self.doStart(params)
            except self.frontend.TerminationRequested:
                print "User requested termination, exiting..."
            except:
                details = _("Exception: %s\nValue: %s\nBacktrace:\n%s\n") % (sys.exc_info()[0], sys.exc_info()[1], "".join(traceback.format_tb(sys.exc_info()[2])))
                print "%s" % details.encode("utf_8")
                self.frontend.performAction("showFatalError", {"message": _("Fatal error occurred in background tasks.\nPlease send bug report to author, attaching full output from console."), "details" : details})
                raise
        finally:
            safeDirectoryRemove(self.mainTempDir)
            self.frontend.backendTerminated()
    def doStart(self, params):
        command = self.frontend.waitForInput()
        firstLoop = True
        while True:
            if firstLoop == True:
                firstLoop = False
            else:
                self.frontend.performAction(
                    "setNextPage", 
                    {
                        "pageName": "simpleChoice",
                        "pageTitle" : _("Wizard failure"),
                        "text": _("One of the operations failed.\nDo you want to restart the wizard from the beginning or quit the wizard?"),
                        "choices": {
                            "restart": {"text": _("Restart wizard"), "checked" : True },
                            "quit": {"text": _("Quit wizard") }
                            }
                        }
                    )
                restartOutput = self.frontend.waitForInput()
                
                if restartOutput["choice"] == "quit":
                    self.frontend.performAction(
                        "setNextPage", 
                        {
                            "pageName": "simpleMessage",
                            "pageTitle" : _("Wizard failure"),
                            "nextEnabled" : False,
                            "finishEnabled" : True,
                            "text": _("<p>Thank you for using NeoGui.</p><p>If you think the behaviour spotted is due to a bug in application, please report bug to the author, along with output from console.</p><p>Click \"Finish\" to quit wizard.</p>"),
                            }
                        )
                    return
                else:
                    pass #Move to first wizard step
            
            modemType = self.detectModem()
            if modemType == None:
                self.frontend.performAction("setNextPage", {"pageName": "manualModemSelect"})
                modemTypeOutput = self.frontend.waitForInput()
                modemType = modemTypeOutput["modemType"]
                
            pppPeersTemplate = """noipdefault
defaultroute
user %s
noauth
updetach
usepeerdns
plugin pppoatm.so
0.35
"""
                
            if modemType == "speedTouch":
                if self.configureSpeedtouch() == False:
                    print "Configuring speedTouch failed, restarting from beginning"
                    continue
                modemDriverName = "speedtch"
                self.replugModem(modemDriverName, removeDrivers = True)
                if self.waitForAtmModemSync("/proc/net/atm/speedtch:0") == False:
                    print "Waiting for SpeedTouch modem sync failed, restarting"
                    continue
            elif modemType == "sagemFast800":
                if self.configureSagemFast800() == False:
                    print "Configuring Sagem Fast 800 failed, restarting from beginning"
                    continue
                modemDriverName = "ueagle_atm"
                self.replugModem(modemDriverName)
                if self.waitForSagemModemSync("/proc/net/atm/ueagle-atm:0") == False:
                    print "Waiting for Sagem Fast 800 modem sync failed, restarting"
                    continue
            elif modemType == "sagemE4":
                if self.configureSagemE4() == False:
                    print "Configuring Sagem E4 failed, restarting from beginning"
                    continue
                modemDriverName = "ueagle4_atm"
                self.replugModem(modemDriverName)
                if self.waitForSagemModemSync("/proc/net/atm/ueagle4-atm:0") == False:
                    print "Waiting for Sagem E4 modem sync failed, restarting"
                    continue
            elif modemType == "Zx":
                if self.configureZx852() == False:
                    print "Configuring ZXDSL852 modem failed, restarting from beginning"
                    continue
                modemDriverName = "cxacru"
                self.replugModem(modemDriverName)
                if self.waitForAtmModemSync("/proc/net/atm/cxacru:0") == False:
                    print "Waiting for ZXDSL852 modem sync failed, restarting"
                    continue
            elif modemType == "Zx2":
                if self.configureZx852v2() == False:
                    print "Configuring ZXDSL852v2 modem failed, restarting from beginning"
                    continue
                modemDriverName = "unicorn_usb_eth"
                pppPeersTemplate = """
lock
ipparam ppp0
noipdefault
noauth
defaultroute
user %s
noaccomp
noccp
nobsdcomp
nodeflate
nopcomp
novj
lcp-echo-interval 20
lcp-echo-failure 3
maxfail 25
updetach
persist
holdoff 4
pty "/usr/sbin/pppoe -I dsl0 -m 1452"
"""
                self.replugModem(modemDriverName)
                if self.waitForZx852v2ModemSync() == False:
                    print "Waiting for ZXDSL852v2 modem sync failed, restarting"
                    continue
            else:
                self.frontend.performAction(
                    "setNextPage", 
                    {
                        "pageName": "simpleMessage",
                        "pageTitle" : _("Modem not handled yet"),
                        "nextEnabled" : True,
                        "text": _("<p>Sorry, this modem type is not yet handled.</p>"),
                        }
                    )
                self.frontend.waitForInput()
                continue
            #Check if registration is needed
            self.frontend.performAction(
                "setNextPage", 
                {
                    "pageName": "isRegistrationNeeded"
                    }
                )
            
            registrationNeededOutput = self.frontend.waitForInput()
            if registrationNeededOutput["registrationNeeded"] == True:
                if self.connectForRegistration(pppPeersTemplate) == False:
                    self.frontend.waitForInput()
                    continue
                self.frontend.performAction(
                    "setNextPage", 
                    {
                        "pageName": "registration",
                        "registrationUrl": "http://rejestracja.neostrada.pl"
                        }
                    )
                self.frontend.waitForInput()
            self.frontend.performAction(
                "setNextPage", 
                {
                    "pageName": "usernameAndPassword"
                    }
                )
            usernameAndPasswordOutput = self.frontend.waitForInput()
            self.setupPppConnection(
                usernameAndPasswordOutput["username"], 
                usernameAndPasswordOutput["password"],
                pppPeersTemplate
                )
            if self.connectToNetwork() == False:
                #failure
                print "Connecting to network failed"
                self.frontend.waitForInput()
                continue
            if self.askForConnectionStartupAtBoot() == False:
                print "Setting up persistent connection failed"
                continue
            print "Successfully set up"
            break
        #Success
        self.frontend.performAction(
            "setNextPage", 
            {
                "pageName": "finish",
                "finishEnabled": True
                }
            )

        print "Exiting backend thread"
