#!/usr/bin/env python
import sys
from qt import *
import threading
import qt
import subprocess
import os
import config_wizard
import gettext
import neogui_lib
import neogui_qt_lib


def messageBoxWithLog(
    parent,
    caption,
    message,
    details
    ):
    messageBox = neogui_qt_lib.MessageBoxWithLog(
        parent,
        caption,
        message,
        details
        )
    messageBox.show()
    return messageBox.exec_loop()

class NeoGuiQtWindow(config_wizard.NeostradaConfigWizardUI):
    def __init__(self, frontend, backend, *args):
        self.initWidgetNamePagesNames()
        self.pages={}
        apply(config_wizard.NeostradaConfigWizardUI.__init__, (self,) + args)
        self.setCaption(_("Neostrada configuration wizard"))
        self.simpleChoiceButtons = {}
        self.frontend = frontend
        self.backend = backend
        self.backButton().setText(_("Back"))
        self.nextButton().setText(_("Next"))
        self.finishButton().setText(_("Finish"))
        self.cancelButton().setText(_("Cancel"))
        self.helpButton().setText(_("Help"))
        self.connect(
            self.frontend,
            PYSIGNAL("setNextPage"),
            self.slotSetNextPage
            )
        self.connect(
            self.frontend,
            PYSIGNAL("progressInfo"),
            self.slotProgressInfo
            )
        self.connect(
            self.frontend,
            PYSIGNAL("showFatalError"),
            self.slotShowFatalError
            )
        self.connect(
            self.frontend,
            PYSIGNAL("showPopupDialog"),
            self.slotShowPopupDialog
            )
        self.connect(
            self,
            PYSIGNAL("signalGetPageOutput"),
            self.frontend.slotGetPageOutput
            )
        self.connect(
            self.progressDetailsButton,
            qt.SIGNAL("clicked()"),
            self.toggleProgressDetails
            )
        self.connect(
            self.buttonFileDialogFirmware,
            qt.SIGNAL("clicked()"),
            self.openFileDialogFirmware
            )
        self.connect(
            self.firmwarePackageLocation,
            qt.SIGNAL("textChanged( const QString & )"),
            self.checkFirmwareLocation
            )
        self.connect(
            self.buttonGroupSelectModem,
            qt.SIGNAL("clicked(int)"),
            self.enableButtonNext
            )
        self.connect(
            self.buttonOpenBrowser,
            qt.SIGNAL("clicked()"),
            self.openBrowserForRegistration
            )
        self.connect(
            self.buttonGroupSelectFirmwareSource,
            qt.SIGNAL("clicked(int)"),
            self.enableButtonNext
            )
        self.connect(
            self.username,
            qt.SIGNAL("textChanged( const QString & )"),
            self.checkUsernameAndPasswordTyped
            )
        self.connect(
            self.password,
            qt.SIGNAL("textChanged( const QString & )"),
            self.checkUsernameAndPasswordTyped
            )
        self.connect(
            self.forceTerminationButton,
            qt.SIGNAL("clicked()"),
            self.forceBackendTermination
            )
        page0 = qt.QWizard.page(self, 0)
        self.currentPageOptions = {}
        #self.setNextEnabled(page0, False)
        #self.setFinishEnabled(page0, False)
        
        self.backend.startWizard()
    def currentPageName(self):
        for (pageName, page) in self.pages.iteritems():
            if page == self.currentPage():
                return pageName
        raise Exception, "Cannot find current page name, pageIndex=%s" % self.indexOf(self.currentPage())
    def enterManualModemSelectionPage(self):
        if self.buttonGroupSelectModem.selectedId == -1:
            self.setNextEnabled(self.currentPage(), False)
    def enterFirmwareInstallationSourcePage(self):
        self.setNextEnabled(self.currentPage(), True)
    def enterRegistrationPage(self):
        self.setNextEnabled(self.currentPage(), False)
    def enterUsernameAndPasswordPage(self):
        self.setNextEnabled(self.currentPage(), False)
        self.username.setFocus()
        self.checkUsernameAndPasswordTyped()
    def enterFirmwareInstallationFromLocationPage(self):
        self.setNextEnabled(self.currentPage(), False)
        self.checkFirmwareLocation()
    def enterOperationProgressPage(self):
        self.setNextEnabled(self.currentPage(), False)
        self.progressLabel.setText("")
        if self.currentPageOptions.has_key("progressLabel"):
            self.progressLabel.setText(self.currentPageOptions["progressLabel"])
        self.progressLogText = ""
        self.errorsFrame.hide()
        self.progressLog.hide()
    def enterSimpleChoicePage(self):
        self.setNextEnabled(self.currentPage(), True)
        self.simpleChoiceText.setText(self.currentPageOptions.get("text", ""))
        self.simpleChoiceButtons = {}
        for choiceKey, choiceParams in self.currentPageOptions["choices"].iteritems():
            print "Adding choice: %s params=%s" % (choiceKey, repr(choiceParams))
            button = qt.QRadioButton(self.simpleChoiceRadioGroup)
            self.simpleChoiceRadioGroup.layout().addWidget(button)
            button.setText(choiceParams["text"])
            button.setEnabled(choiceParams.get("enabled", True))
            button.setChecked(choiceParams.get("checked", False))
            button.show()
            self.simpleChoiceButtons[choiceKey] = button
    def enterSimpleMessagePage(self):
        self.simpleMessageText.setText(self.currentPageOptions.get("text", ""))
    def enterPlugModemPage(self):
        defaultMessage = _("""<p><u><b>If the modem is not detected:</b></u></p>
<p>- Try unplugging and plugging the modem in.</p>
<p>- Check if any LED on modem is on. If it is off, there is a problem with USB port and modem is not powered - please try connecting to another USB port.</p>""")
        if self.currentPageOptions.get("message", None) != None:
            self.plugModemMessage.setText(self.currentPageOptions.get("message"))
        else:
            self.plugModemMessage.setText(defaultMessage)
        
    def enterTerminationPage(self):
        self.setNextEnabled(self.currentPage(), False)
        self.setFinishEnabled(self.currentPage(), False)
        self.terminationCounter = 0
        self.checkBackendTermination()
        self.terminationTimer = qt.QTimer(self)
        self.connect(
            self.terminationTimer, 
            qt.SIGNAL("timeout()"), 
            self.checkBackendTermination
            )
        self.terminationTimer.start(200) #200 ms timer
        self.frontend.requestBackendTermination()
    def forceBackendTermination(self):
        self.frontend.requestBackendTermination(force = True)
    def checkBackendTermination(self):
        print "Checking if backend terminated"
        if self.frontend.isBackendTerminated() == True:
            print "Backend terminated, closing"
            qt.QWizard.reject(self)
        else:
            print "Backend not terminated, waiting"
            self.terminationCounter = (self.terminationCounter + 1) % 4
            dots = "." * self.terminationCounter
            self.terminationLabel.setText(_("Waiting for background tasks termination") + dots)
    def openBrowserForRegistration(self):
        import subprocess
        #run in background
        self.browserProcess = subprocess.Popen(
            ["/usr/bin/x-www-browser", self.currentPageOptions["registrationUrl"]],
            stdout = file("/dev/null", "w"),
            stderr = file("/dev/null", "w")
            )
        self.setNextEnabled(self.currentPage(), True)
        
    def toggleProgressDetails(self):
        print  "Progress details button clicked"
        if self.progressLog.isHidden():
            self.progressLog.show()
        else:
            self.progressLog.hide()
    def openFileDialogFirmware(self):
        location = qt.QFileDialog.getOpenFileName(
            os.path.dirname(os.path.abspath(sys.argv[0])), #startWith
            self.currentPageOptions["firmwareFileWildcard"], #filter
            self, #parent
            "openFileDialogFirmware", #name = 0
            _("Select firmware package location") #caption
            )
        self.firmwarePackageLocation.setText(location)
    def checkFirmwareLocation(self):
        filePath = str(self.firmwarePackageLocation.text())
        if os.path.isfile(filePath):
            self.setNextEnabled(self.currentPage(), True)
        else:
            self.setNextEnabled(self.currentPage(), False)
    def checkUsernameAndPasswordTyped(self):
        if len(self.username.text()) > 0 and len(self.password.text()) > 0:
            self.setNextEnabled(self.currentPage(), True)
        else:
            self.setNextEnabled(self.currentPage(), False)
    def enableButtonNext(self, id):
        self.setNextEnabled(self.currentPage(), True)
    def getManualModemSelectionPageOutput(self):
        if self.buttonSpeedtouch.isChecked():
            modemType = "speedTouch"
        elif self.buttonSagemFast.isChecked():
            modemType = "sagemFast800"
        elif self.buttonSagemFastE4.isChecked():
            modemType = "sagemE4"
        elif self.buttonZx.isChecked():
            modemType = "Zx"
        elif self.buttonZx2.isChecked():
            modemType = "Zx2"
        else:
            raise Exception, "Unhandled radio button checked"
        return {"modemType": modemType}
    def getSimpleChoicePageOutput(self):
        for choiceLabel, choiceButton in self.simpleChoiceButtons.iteritems():
            if choiceButton.isChecked():
                chosenButton = choiceLabel
            choiceButton.close()
        return {"choice": chosenButton}
            
    def getIsRegistrationNeededPageOutput(self):
        if self.buttonNoUsername.isChecked():
            registrationNeeded = True
        elif self.buttonHasUsername.isChecked():
            registrationNeeded = False
        else:
            raise Exception, "Unhandled radio button checked"
        return {"registrationNeeded": registrationNeeded}
    def getUsernameAndPasswordPageOutput(self):
        return {"username": self.username.text(), "password" : self.password.text()}
    def getFirmwareSourcePageOutput(self):
        if self.buttonFirmwareSourceBundled.isChecked():
            firmwareSource = "bundled"
        elif self.buttonFirmwareSourceLocation.isChecked():
            firmwareSource = "location"
        elif self.buttonFirmwareSourceInternet.isChecked():
            firmwareSource = "Internet"
        elif self.buttonFirmwareSourceCD.isChecked():
            firmwareSource = "InstallCD"
        else:
            raise Exception, "Unhandled radio button checked"
        return {"firmwareSource": firmwareSource}
    def getFirmwareInstallationFromLocationPageOutput(self):
        fileLocation = str(self.firmwarePackageLocation.text())
        return {"firmwareFileLocation": fileLocation}
    def getPageOutput(self):
        pageName = self.currentPageName()
        if pageName == "manualModemSelect":
            return self.getManualModemSelectionPageOutput()
        elif pageName == "firmwareInstallationSource":
            return self.getFirmwareSourcePageOutput()
        elif pageName == "firmwareInstallationFromLocation":
            return self.getFirmwareInstallationFromLocationPageOutput()
        elif pageName == "isRegistrationNeeded":
            return self.getIsRegistrationNeededPageOutput()
        elif pageName == "usernameAndPassword":
            return self.getUsernameAndPasswordPageOutput()
        elif pageName == "simpleChoice":
            return self.getSimpleChoicePageOutput()
        else:
            return {}
    def reject(self):
        options = {"pageName": "terminating"}
        self.slotSetNextPage(options)
    def showPage(self, page):
        qt.QWizard.showPage(self, page)
        pageName = self.currentPageName()
        print "Entering page: %s" % pageName
        if pageName == "manualModemSelect":
            self.enterManualModemSelectionPage()
        elif pageName == "firmwareInstallationSource":
            self.enterFirmwareInstallationSourcePage()
        elif pageName == "firmwareInstallationFromLocation":
            self.enterFirmwareInstallationFromLocationPage()
        elif pageName == "actionProgress":
            self.enterOperationProgressPage()
        elif pageName == "registration":
            self.enterRegistrationPage()
        elif pageName == "usernameAndPassword":
            self.enterUsernameAndPasswordPage()
        elif pageName == "simpleChoice":
            self.enterSimpleChoicePage()
        elif pageName == "simpleMessage":
            self.enterSimpleMessagePage()
        elif pageName == "terminating":
            self.enterTerminationPage()
        elif pageName == "plugModem":
            self.enterPlugModemPage()
    def next(self):
        pageOutput = self.getPageOutput()
        print "Emitting signalGetPageOutput, pageOutput=%s" % repr(pageOutput)
        self.emit(PYSIGNAL("signalGetPageOutput"), (pageOutput,))
        #self.setNextEnabled(self.currentPage(), False)
        #self.setFinishEnabled(self.currentPage(), False)
    def initWidgetNamePagesNames(self):
        self.addPageIndex = 0
        self.widgetNamePagesNames = [ "welcome", "modemDetect", "manualModemSelect", "installationMedia",
            "actionProgress", "firmwareInstallationSource", "firmwareInstallationFromLocation",
            "unplugModem", "plugModem",
            "isRegistrationNeeded",
            "registration", "usernameAndPassword", "terminating", "simpleChoice", "simpleMessage", "finish"
            ]
    def addPage(self, pageWidget, pageName):
        if self.addPageIndex < len(self.widgetNamePagesNames):
            pageName = self.widgetNamePagesNames[self.addPageIndex]
        else:
            pageName = "page%s" % self.addPageIndex
        print "addPage: addPageIndex=%s widgetName=%s pageName=%s" % (self.addPageIndex, pageWidget.name(), pageName)
        self.pages[pageName] = pageWidget
        self.addPageIndex += 1
        qt.QWizard.addPage(self, pageWidget, pageName)
    def slotSetNextPage(self, pageOptions):
        print "slotSetNextPage: pageOptions=%s" % repr(pageOptions)
        nextPageName = pageOptions["pageName"]
        selectedPage = None
        for (pageName, page) in self.pages.iteritems():
            if pageName == nextPageName:
                #self.setAppropriate(page, True)
                selectedPage = page
            else:
                #self.setAppropriate(page, False)
                pass
        self.currentPageOptions = pageOptions
        #qt.QWizard.next(self)
        self.showPage(selectedPage)
        if pageOptions.has_key("nextEnabled"):
            self.setNextEnabled(self.currentPage(), pageOptions["nextEnabled"])
        if pageOptions.has_key("finishEnabled"):
            self.setFinishEnabled(self.currentPage(), pageOptions["finishEnabled"])
        if self.currentPageOptions.has_key("pageTitle"):
            self.setTitle(self.currentPage(), self.currentPageOptions["pageTitle"])
        self.setBackEnabled(self.currentPage(), False) #not supported yet
    def slotShowFatalError(self, options):
        messageBoxWithLog(
            self, 
            _("Fatal error"), 
            _("Fatal error occurred:\n%s\nConfiguration wizard will now quit.") % options["message"],
            options["details"]
            )
        self.close()
    def slotShowPopupDialog(self, options):
        result = qt.QMessageBox.question(
            self, #parent
            options["caption"], 
            options["message"],
            options.get("button0Text", None),
            options.get("button1Text", None),
            options.get("button2Text", qt.QString()),
            options.get("defaultButtonNumber", 0)
            )
        pageOutput = {"result" : result}
        self.emit(PYSIGNAL("signalGetPageOutput"), (pageOutput,))
    def slotProgressInfo(self, info):
        if info.has_key("text"):
            self.progressState.setText(info["text"])
        if info.has_key("details"):
            self.progressLogText += info["details"]
            self.progressLog.append(info["details"])
        if info.has_key("nextEnabled"):
            self.setNextEnabled(self.currentPage(), info["nextEnabled"])
        if info.has_key("status"):
            status = info["status"]
            if status == False:
                self.errorsFrame.show()
                self.setNextEnabled(self.currentPage(), True)


def main(args):
    app=QApplication(args)
    frontend = neogui_qt_lib.NeoGuiQtFrontend()
    import neogui_config_backend
    backend = neogui_config_backend.NeoGuiConfigBackend(frontend)
    win=NeoGuiQtWindow(frontend, backend)
    win.setIcon(qt.QPixmap(neogui_lib.thisScriptDirectory() + "/img/connect_established.png"))

    win.show()
    app.connect(app, SIGNAL("lastWindowClosed()"),
                app, SLOT("quit()"))
    app.exec_loop()

if __name__=="__main__":
    gettext.install('neogui', neogui_lib.thisScriptDirectory() + "/translations" , unicode=1)
    main(sys.argv)