#!/usr/bin/env python
import sys
from qt import *
import threading
import qt
import subprocess
import os
import gettext
import neogui_lib
import neogui_qt_lib
import neogui_config_backend

class ConnectionWindow(neogui_qt_lib.MessageBoxWithLog):
    def __init__(
        self, 
        frontend,
        parent,
        *args
        ):
        apply(neogui_qt_lib.MessageBoxWithLog.__init__, (
            self, 
            parent, 
           _("Connecting to network"),
            "",
            "",
            ) + args)
        self.frontend = frontend
        self.connect(
            self.frontend,
            PYSIGNAL("progressInfo"),
            self.slotProgressInfo
            )
        self.buttonOk.setText(_("Close"))
        self.buttonOk.setEnabled(False)
        self.resize(200,150)
    def slotProgressInfo(self, info):
        if info.has_key("text"):
            self.slotUpdateMessage(info["text"])
        if info.has_key("details"):
            self.slotAppendDetails(info["details"])
        #if info.has_key("nextEnabled"):
            #self.setNextEnabled(self.currentPage(), info["nextEnabled"])
        if info.has_key("status"):
            self.buttonOk.setEnabled(True)
            if info["status"] == True:
                self.slotUpdateMessage(_("Modem connected, you can close this window"))
    def accept(self):
        print "Slot accept"
        self.frontend.slotGetPageOutput({})
        neogui_qt_lib.MessageBoxWithLog.accept(self)

class NeoGuiQtWindow(QMainWindow):
    def __init__(self, frontend, backend, *args):
        apply(QMainWindow.__init__, (self,) + args)
        self.updateThread = neogui_lib.ConnectionUpdateThread()
        self.updateThread.start()
        self.iconConnectingActive = False
        
        self.frontend = frontend
        self.backend = backend
        self.mainView=QVBox(self)
        self.setCentralWidget(self.mainView)
        self.buttonConnect = qt.QPushButton(self.mainView)
        self.buttonConnect.setText(_("Connect"))
        self.buttonDisconnect = qt.QPushButton(self.mainView)
        self.buttonDisconnect.setText(_("Disconnect"))
        self.buttonConfigure = qt.QPushButton(self.mainView)
        self.buttonConfigure.setText(_("Configure connection"))
        self.buttonQuit = qt.QPushButton(self.mainView)
        self.buttonQuit.setText(_("Quit"))
        self.labelModemStatus = qt.QLabel(self.mainView)
        self.labelConnectionStatus = qt.QLabel(self.mainView)
        self.labelConnectionTime = qt.QLabel(self.mainView)
        self.previousConnectionSetup = None
        self.iconConnecting = qt.QPixmap(neogui_lib.thisScriptDirectory() + "/img/connect_creating.png")
        self.iconConnected = qt.QPixmap(neogui_lib.thisScriptDirectory() + "/img/connect_established.png")
        self.iconNotConnected = qt.QPixmap(neogui_lib.thisScriptDirectory() + "/img/connect_no.png")
        
        self.connect(
            self.buttonQuit,
            qt.SIGNAL("clicked()"),
            qt.SLOT("close()")
            )
        self.connect(
            self.buttonConnect,
            qt.SIGNAL("clicked()"),
            self.connectToNetwork
            )
        self.connect(
            self.buttonDisconnect,
            qt.SIGNAL("clicked()"),
            self.disconnectFromNetwork
            )
        self.connect(
            self.buttonConfigure,
            qt.SIGNAL("clicked()"),
            self.configureConnection
            )
        self.reloadState()
        self.reloadTimer = qt.QTimer(self)
        self.connect(
            self.reloadTimer, 
            qt.SIGNAL("timeout()"), 
            self.reloadState
            )
        
        self.reloadTimer.start(1000) #1000 ms timer
    def isConnectionSetUp(self):
        #FIXME: replace it with real checking what is the status of modem line, etc.
        if os.path.exists("/etc/ppp/peers/neostrada"):
            return True
        else:
            return False
    def reloadState(self):
        self.updateThread.lock()
        receivedBytes = self.updateThread.receivedBytes
        onoff = self.updateThread.onoff
        address = self.updateThread.address
        gateway = self.updateThread.gateway
        rxtx = self.updateThread.rxtx
        connectionTime = self.updateThread.connectionTime
        self.updateThread.unlock()
        if False:
            print "Current state:"
            print receivedBytes
            print onoff
            print address
            print gateway
            print rxtx
            print connectionTime
        
        connectionSetup = self.isConnectionSetUp()
        self.previousConnectionSetup = connectionSetup
        if connectionSetup == True:
            modemStatus = _("Configured")
        else:
            modemStatus = _("Not configured")
        #FIXME: add to backend checking if connection is running
        connectionRunning = onoff
        self.labelModemStatus.setText(_("Modem status: %s") % modemStatus)
        #self.buttonConnect.setEnabled(connectionSetup)
        if onoff == True and address == "":
            status = "connecting"
            connectionStatus = _("Connecting")
            if self.iconConnectingActive:
                self.setIcon(self.iconNotConnected)
            else:
                self.setIcon(self.iconConnecting)
            self.iconConnectingActive = not self.iconConnectingActive
        elif onoff == True and address != "":
            status = "connected"
            connectionStatus = _("Connected")
            self.setIcon(self.iconConnected)
        else:
            status = "not_connected"
            connectionStatus = _("Not connected")
            self.setIcon(self.iconNotConnected)
            
        self.labelConnectionStatus.setText(_("Connection status: %s") % connectionStatus)
        self.labelConnectionTime.setText(_("Connection time: %s") % connectionTime)
        
        if connectionSetup and status != "not_connected":
            self.buttonDisconnect.setEnabled(True)
        else:
            self.buttonDisconnect.setEnabled(False)
            
        if connectionSetup and status == "not_connected":
            self.buttonConnect.setEnabled(True)
        else:
            self.buttonConnect.setEnabled(False)
        self.buttonConfigure.setEnabled(True)
    def disconnectFromNetwork(self):
        self.killProcess = subprocess.Popen(
            ["poff", "neostrada"]
            )
        self.buttonDisconnect.setEnabled(False)
    def connectToNetwork(self):
        connectionFrontend = neogui_qt_lib.NeoGuiQtFrontend()
        connectionWindow = ConnectionWindow(
            connectionFrontend,
            self
            )
        connectionWindow.show()
        connectionBackend = neogui_config_backend.NeoGuiConfigBackend(connectionFrontend)
        connectionBackend.startConnection()
        self.buttonConnect.setEnabled(False)
        
    def configureConnection(self):
        pathToConfigureScript = neogui_lib.thisScriptDirectory() + "/neogui_config_qt.py"
        print "pathToConfigureScript=", pathToConfigureScript
        self.browserProcess = subprocess.Popen(
            [systemEnvironment.getSudoGuiPath(), "python", pathToConfigureScript]
            )
        self.reloadState()
    def quitApp(self):
        global app
        self.updateThread.terminate()
        qt.QApplication.quit(app)

def main(args):
    global app
    app=QApplication(args)
    frontend = neogui_qt_lib.NeoGuiQtFrontend()
    backend = neogui_config_backend.NeoGuiConfigBackend(frontend)
    win=NeoGuiQtWindow(frontend, backend)
    win.setCaption(_("NeoGui"))
    win.setIcon(qt.QPixmap(neogui_lib.thisScriptDirectory() + "/img/connect_no.png"))
    win.show()
    app.connect(app, SIGNAL("lastWindowClosed()"),
                win.quitApp)
    app.exec_loop()

if __name__=="__main__":
    gettext.install('neogui', neogui_lib.thisScriptDirectory() + "/translations" , unicode=1)
    systemEnvironment = neogui_lib.SystemEnvironment()
    main(sys.argv)